<?php

namespace Drupal\editor_note\Form;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Drupal\Core\Url;

use Drupal\editor_note\EditorNoteHelperService;
use Drupal\editor_note\Entity\EditorNote;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements the ModalForm form controller.
 *
 * This example demonstrates implementation of a form that is designed to be
 * used as a modal form.  To properly display the modal the link presented by
 * the \Drupal\form_api_example\Controller\Page page controller loads the Drupal
 * dialog and ajax libraries.  The submit handler in this class returns ajax
 * commands to replace text in the calling page after submission .
 *
 * @see \Drupal\Core\Form\FormBase
 */
class ModalForm extends FormBase {

  /**
   * The messenger.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The editor note helper.
   *
   * @var \Drupal\editor_note\EditorNoteHelperService
   */
  protected $editorNoteHelper;

  /**
   * Form constructor.
   *
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger.
   * @param \Drupal\editor_note\EditorNoteHelperService $editorNoteHelper
   *   Editor note helpers.
   */
  public function __construct(MessengerInterface $messenger, EditorNoteHelperService $editorNoteHelper) {
    $this->messenger = $messenger;
    $this->editorNoteHelper = $editorNoteHelper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('messenger'),
      $container->get('editor_note.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'modal_form';
  }

  /**
   * Helper method so we can have consistent dialog options.
   *
   * @return string[]
   *   An array of jQuery UI elements to pass on to our dialog form.
   */
  protected static function getDataDialogOptions() {
    return [
      'width' => '50%',
    ];
  }

  /**
   * Returns current EditorNote entity.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   *
   * @return \Drupal\editor_note\Entity\EditorNote|null
   *   Editor note entity.
   */
  protected function getEditorNoteEntity(FormStateInterface $form_state) {
    $editor_note = NULL;
    $build_info = $form_state->getBuildInfo();
    if (!empty($build_info['args'][0])) {
      if ($build_info['args'][0] instanceof EditorNote) {
        $editor_note = $build_info['args'][0];
      }
    }
    return $editor_note;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, EditorNote $editor_note = NULL, $nojs = NULL) {
    // Add the core AJAX library.
    $form['#attached']['library'][] = 'core/drupal.ajax';

    // Add a link to show this form in a modal dialog if we're not already in
    // one.
    if ($nojs == 'nojs') {
      $form['use_ajax_container'] = [
        '#type' => 'details',
        '#open' => TRUE,
      ];
      $form['use_ajax_container']['description'] = [
        '#type' => 'item',
        '#markup' => $this->t('In order to show a modal dialog by clicking on a link, that link has to have class <code>use-ajax</code> and <code>data-dialog-type="modal"</code>. This link has those attributes.'),
      ];
      $form['use_ajax_container']['use_ajax'] = [
        '#type' => 'link',
        '#title' => $this->t('Edit in modal.'),
        '#url' => Url::fromRoute('editor_note.modal_form', ['editor_note' => $editor_note->id(), 'nojs' => 'ajax']),
        '#attributes' => [
          'class' => ['use-ajax'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => json_encode(static::getDataDialogOptions()),
          // Add this id so that we can test this form.
          'id' => 'ajax-example-modal-link',
        ],
      ];

      $form['use_ajax_container']['delete'] = [
        '#type' => 'link',
        '#title' => $this->t('Delete in modal.'),
        '#url' => Url::fromRoute('editor_note.confirm_delete_editor_note_form', ['nojs' => 'ajax']),
        '#attributes' => [
          'class' => ['use-ajax'],
          'data-dialog-type' => 'modal',
          'data-dialog-options' => json_encode(static::getDataDialogOptions()),
          // Add this id so that we can test this form.
          'id' => 'ajax-example-modal-link',
        ],
      ];
    }

    // This element is responsible for displaying form errors in the AJAX
    // dialog.
    if ($nojs == 'ajax') {
      $form['status_messages'] = [
        '#type' => 'status_messages',
        '#weight' => -999,
      ];
    }

    $note_value = $editor_note->note->value;
    $user_input = $form_state->getUserInput();
    if (!empty($user_input['editor_note'])) {
      $note_input = $user_input['editor_note'];
      if (isset($note_input['value'])) {
        $note_value = $note_input['value'];
      }
    }
    $form['editor_note'] = [
      '#type' => 'textarea',
      '#default_value' => $note_value,
      '#required' => TRUE,
      '#title' => $this->t('Update note'),
      '#weight' => '0',
    ];

    // Group submit handlers in an actions element with a key of "actions" so
    // that it gets styled correctly, and so that other modules may add actions
    // to the form.
    $form['actions'] = [
      '#type' => 'actions',
    ];

    // Add a submit button that handles the submission of the form.
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Update'),
      '#ajax' => [
        'callback' => '::ajaxSubmitForm',
        'event' => 'click',
      ],
    ];

    // Set the form to not use AJAX if we're on a nojs path. When this form is
    // within the modal dialog, Drupal will make sure we're using an AJAX path
    // instead of a nojs one.
    if ($nojs == 'nojs') {
      unset($form['actions']['submit']['#ajax']);
    }

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $input = $form_state->getValue('editor_note');
    if (empty($input)) {
      $form_state->setError($form['editor_note'], $this->t('Note text is empty.'));
    }
  }

  /**
   * Updates EditorNote entity.
   *
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $title = $form_state->getValue('title');
    if ($editor_note = $this->getEditorNoteEntity($form_state)) {
      $note = $form_state->getValue('editor_note');
      $new_note = [
        'value' => Xss::filterAdmin($note),
        // @todo: implement support for text formats.
        'format' => 'plain_text',
      ];
      $editor_note->set('note', $new_note);
      $editor_note->save();
    }

    $this->messenger->addMessage(
      $this->t('Submit handler: You specified a title of @title.', ['@title' => $title])
    );
  }

  /**
   * Implements the submit handler for the modal dialog AJAX call.
   *
   * @param array $form
   *   Render array representing from.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Current form state.
   *
   * @return \Drupal\Core\Ajax\AjaxResponse
   *   Array of AJAX commands to execute on submit of the modal form.
   */
  public function ajaxSubmitForm(array &$form, FormStateInterface $form_state) {
    // We begin building a new ajax reponse.
    $response = new AjaxResponse();

    // If the user submitted the form and there are errors, show them the
    // input dialog again with error messages. Since the title element is
    // required, the empty string wont't validate and there will be an error.
    if ($form_state->getErrors()) {
      // If there are errors, we can show the form again with the errors in
      // the status_messages section.
      $form['status_messages'] = [
        '#type' => 'status_messages',
        '#weight' => -10,
      ];
      $response->addCommand(new OpenModalDialogCommand($this->t('Errors'), $form, static::getDataDialogOptions()));
    }
    // If there are no errors, show the output dialog.
    else {
      // We don't want any messages that were added by submitForm().
      $this->messenger->deleteAll();
      $editor_note = $this->getEditorNoteEntity($form_state);
      // This will be the contents for the modal dialog.
      $content = [
        '#type' => 'item',
        '#markup' => $this->t('Note has been updated and saved.'),
      ];
      // Add the OpenModalDialogCommand to the response. This will cause Drupal
      // AJAX to show the modal dialog. The user can click the little X to close
      // the dialog.
      $response->addCommand(new OpenModalDialogCommand($this->t('Update selected item'), $content, static::getDataDialogOptions()));

      // Refresh editor note table.
      $field_config = $this->editorNoteHelper->getNoteFieldDefinition($editor_note);
      $host_entity_id = $editor_note->entity_id->target_id;
      $field_name = $editor_note->field_machine_name->value;
      $widget_settings = $this->editorNoteHelper->getNoteFieldWidgetSettings($editor_note);
      $notes = $this->editorNoteHelper->getNotesByEntityAndField($host_entity_id, $field_name, $widget_settings);
      $table = $this->editorNoteHelper->generateTable($field_config, $notes, TRUE);
      $response->addCommand(new ReplaceCommand('#formatted_notes_' . $field_name, $table));
    }

    // Finally return our response.
    return $response;
  }

}
