<?php

namespace Drupal\editor_note\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\editor_note\EditorNoteHelperService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Editor note widget.
 *
 * @FieldWidget(
 *   id = "editor_note_widget",
 *   label = @Translation("Editor Note"),
 *   module = "editor_note",
 *   field_types = {
 *     "editor_note_item"
 *   }
 * )
 */
class EditorNoteWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  use StringTranslationTrait;

  /**
   * The editor note helper.
   *
   * @var \Drupal\editor_note\EditorNoteHelperService
   */
  protected $editorNoteHelper;

  /**
   * Constructs a CountryDefaultWidget object.
   *
   * @param string $plugin_id
   *   The plugin_id for the formatter.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Field\FieldDefinitionInterface $field_definition
   *   The definition of the field to which the formatter is associated.
   * @param array $settings
   *   The formatter settings.
   * @param array $third_party_settings
   *   Any third party settings settings.
   * @param \Drupal\editor_note\EditorNoteHelperService $editorNoteHelper
   *   Editor note helpers.
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, EditorNoteHelperService $editorNoteHelper) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);

    $this->editorNoteHelper = $editorNoteHelper;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('editor_note.helper')
    );
  }

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    $formats = filter_formats();
    $format = reset($formats);

    return [
      'formatted' => FALSE,
      'default_format' => $format->id(),
      'limit' => 10,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = parent::settingsForm($form, $form_state);

    // @todo: implement support for text formats.
    $element['formatted'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Formatted'),
      '#default_value' => $this->getSetting('formatted'),
      '#weight' => -1,
      '#disabled' => TRUE,
    ];

    $options = [];
    foreach (filter_formats() as $format) {
      $options[$format->id()] = $format->label();
    }

    $field_name = $this->fieldDefinition->getName();
    $element['default_format'] = [
      '#type' => 'select',
      '#title' => $this->t('Default format'),
      '#options' => $options,
      '#default_value' => $this->getSetting('default_format'),
      '#states' => [
        'visible' => [
          ':input[name="fields[' . $field_name . '][settings_edit_form][settings][formatted]"]' => ['checked' => TRUE],
        ],
      ],
    ];

    $element['limit'] = [
      '#type' => 'number',
      '#title' => $this->t('Display a specified number of items.'),
      '#description' => $this->t('Display a specified number of items. Set 0 for no limit.'),
      '#default_value' => $this->getSetting('limit'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = parent::settingsSummary();

    $is_formatted = $this->getSetting('formatted');
    $label = $is_formatted ? $this->t('Yes') : $this->t('No');
    $summary[] = $this->t('Formatted: @formatted', ['@formatted' => $label]);

    if ($is_formatted) {
      $format = filter_formats()[$this->getSetting('default_format')];
      $summary[] = $this->t('Default format: @format', ['@format' => $format->label()]);
    }

    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $entity = $items->getEntity();
    $wrapper_id = 'editor-notes-' . $delta;
    if (!$entity->hasField('path')) {
      $access = $entity->access('edit');
    }
    else {
      $access = $entity->get('path')->access('edit');
    }
    $element = [
      '#type' => 'details',
      '#title' => $this->t('Editor Notes'),
      '#element_validate' => [
        [$this, 'validate'],
      ],
      '#access' => $access,
      '#attributes' => [
        'class' => ['editor-notes-form'],
        'id' => $wrapper_id,
      ],
    ];

    $widget_settings = $this->getSettings();
    $notes = $this->editorNoteHelper->getNotesByEntityAndField($entity->id(), $this->fieldDefinition->getName(), $widget_settings);
    if ($notes) {
      $element['table'] = $this->editorNoteHelper->generateTable($this->fieldDefinition, $notes, TRUE);
    }

    // Determine should it be formatted field or not.
    if ($this->getSetting('formatted')) {
      $element['editor_notes'] = [
        '#type' => 'text_format',
        '#format' => $this->getSetting('default_format'),
        '#default_value' => '',
      ];
    }
    else {
      $element['editor_notes'] = [
        '#type' => 'textarea',
        '#default_value' => '',
      ];
    }

    // Add "Add Note" button to only existed entities.
    if (!$form_state->getFormObject()->getEntity()->isNew()) {
      $element['add_note'] = [
        '#type' => 'button',
        '#value' => $this->t('Add Note'),
        '#ajax' => [
          'callback' => [$this, 'onNoteSubmit'],
          'wrapper' => $wrapper_id,
        ],
      ];
    }

    return $element;
  }

  /**
   * Ajax callback for 'Add Note button'.
   *
   * @param array $form
   *   The form structure where widgets are being attached to. This might be a
   *   full form structure, or a sub-element of a larger form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   Form element.
   */
  public function onNoteSubmit(array $form, FormStateInterface $form_state) {
    $button = $form_state->getTriggeringElement();
    $element =& NestedArray::getValue($form, array_slice($button['#array_parents'], 0, -1));
    $host_entity = $form_state->getFormObject()->getEntity();
    $field_name = $button['#array_parents'][0];
    // @todo: implement support for text formats.
    $format = 'plain_text';

    // Get value depend on formatted field or not.
    if ($this->getSetting('formatted')) {
      $value = $element['editor_notes']['value']['#value'];
      $format = $element['editor_notes']['value']['#format'];
    }
    else {
      $value = $element['editor_notes']['#value'];
    }

    // Save Editor Note entity.
    $this->editorNoteHelper->createNote($host_entity, $field_name, $value, $format);
    $notes = $this->editorNoteHelper->getNotesByEntityAndField($host_entity->id(), $field_name, $this->getSettings());
    // Rebuild Editor Notes table.
    $table = $this->editorNoteHelper->generateTable($this->fieldDefinition, $notes, TRUE);

    // Update some element values.
    $element['table'] = $table;
    $element['#open'] = TRUE;

    if ($this->getSetting('formatted')) {
      $element['editor_notes']['value']['#value'] = '';
    }
    else {
      $element['editor_notes']['#value'] = '';
    }

    return $element;
  }

  /**
   * Validate the editor note field.
   *
   * @param array $element
   *   Element array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   Form State object.
   */
  public function validate(array $element, FormStateInterface $form_state) {
    if ($this->getSetting('formatted')) {
      $value = $element['editor_notes']['value']['#value'];
    }
    else {
      $value = $element['editor_notes']['#value'];
    }
    $form_state->setValueForElement($element, ['value' => $value]);
  }

}
